@@ -1,7 +1,7 @@
 #
 # Monitorix - A lightweight system monitoring tool.
 #
-# Copyright (C) 2005-2016 by Jordi Sanfeliu <jordi@fibranet.cat>
+# Copyright (C) 2005-2019 by Jordi Sanfeliu <jordi@fibranet.cat>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -133,6 +133,7 @@ sub httpd_setup {
 	open(OUT, ">> " . $config->{httpd_builtin}->{log_file});
 	close(OUT);
 	chown($uid, $gid, $config->{httpd_builtin}->{log_file});
+	chmod(0600, $config->{httpd_builtin}->{log_file});
 
 	setgid($gid);
 	if(getgid() != $gid) {
@@ -155,9 +156,8 @@ sub httpd_setup {
 			logger("$myself: '$config->{httpd_builtin}->{auth}->{htpasswd}' $!");
 		}
 	} else {
-		if(!grep {$_ eq $config->{httpd_builtin}->{host}}
-			("localhost", "127.0.0.1")) {
-			logger("WARNING: the built-in HTTP server has authentication disabled.");
+		if(!grep {$_ eq $config->{httpd_builtin}->{host}} ("localhost", "127.0.0.1")) {
+			logger("WARNING: the HTTP built-in server has authentication disabled.");
 		}
 	}
 
