@@ -1,7 +1,7 @@
 #
 # Monitorix - A lightweight system monitoring tool.
 #
-# Copyright (C) 2005-2017 by Jordi Sanfeliu <jordi@fibranet.cat>
+# Copyright (C) 2005-2019 by Jordi Sanfeliu <jordi@fibranet.cat>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -164,22 +164,37 @@ sub gensens_update {
 				my $timeintvl = trim($al[0]);
 				my $threshold = trim($al[1]);
 				my $script = trim($al[2]);
-	
-				if(!$threshold || $val < $threshold) {
-					$config->{gensens_hist_alerts}->{$str} = 0;
-				} else {
-					if(!$config->{gensens_hist_alerts}->{$str}) {
-						$config->{gensens_hist_alerts}->{$str} = time;
-					}
-					if($config->{gensens_hist_alerts}->{$str} > 0 && (time - $config->{gensens_hist_alerts}->{$str}) >= $timeintvl) {
-						if(-x $script) {
-							logger("$myself: alert on Generic Sensor ($str): executing script '$script'.");
-							system($script . " " . $timeintvl . " " . $threshold . " " . $val);
+				my $when = lc(trim($al[3] || ""));
+				my @range = split('-', $threshold);
+				$when = "above" if !$when;	# 'above' is the default
+				$threshold = 0 if !$threshold;
+				if(scalar(@range) == 1) {
+					if($when eq "above" && $val < $threshold) {
+						$config->{gensens_hist_alerts}->{$str} = 0;
+					} elsif($when eq "below" && $val > $threshold) {
+						$config->{gensens_hist_alerts}->{$str} = 0;
+					} else {
+						if($when eq "above" || $when eq "below") {
+							if(!$config->{gensens_hist_alerts}->{$str}) {
+								$config->{gensens_hist_alerts}->{$str} = time;
+							}
+							if($config->{gensens_hist_alerts}->{$str} > 0 && (time - $config->{gensens_hist_alerts}->{$str}) >= $timeintvl) {
+								if(-x $script) {
+									logger("$myself: alert on Generic Sensor ($str): executing script '$script'.");
+									system($script . " " . $timeintvl . " " . $threshold . " " . $val . " " . $when);
+								} else {
+									logger("$myself: ERROR: script '$script' doesn't exist or don't has execution permissions.");
+								}
+								$config->{gensens_hist_alerts}->{$str} = time;
+							}
 						} else {
-							logger("$myself: ERROR: script '$script' doesn't exist or don't has execution permissions.");
+							logger("$myself: ERROR: invalid when value '$when'");
 						}
-						$config->{gensens_hist_alerts}->{$str} = time;
 					}
+				} elsif(scalar(@range) == 2) {
+					logger("$myself: range values are not supported yet.");
+				} else {
+					logger("$myself: ERROR: invalid threshold value '$threshold'");
 				}
 			}
 
