@@ -1,7 +1,7 @@
 #
 # Monitorix - A lightweight system monitoring tool.
 #
-# Copyright (C) 2005-2014 by Jordi Sanfeliu <jordi@fibranet.cat>
+# Copyright (C) 2005-2017 by Jordi Sanfeliu <jordi@fibranet.cat>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,15 +24,20 @@ use strict;
 use warnings;
 use Monitorix;
 use RRDs;
-use POSIX qw(strftime);
+use POSIX qw(strftime setlocale LC_ALL);
 use Exporter 'import';
 our @EXPORT = qw(port_init port_update port_cgi);
 
+# Force a standard locale
+$ENV{LANG} = "";
+setlocale(LC_ALL, "C");
+
 sub port_init {
 	my $myself = (caller(0))[3];
 	my ($package, $config, $debug) = @_;
 	my $rrd = $config->{base_lib} . $package . ".rrd";
 	my $port = $config->{port};
+	my $cmd;
 
 	my $info;
 	my @ds;
@@ -45,6 +50,8 @@ sub port_init {
 	my @max;
 	my @last;
 
+	my $table = $config->{ip_default_table};
+
 	if(-e $rrd) {
 		$info = RRDs::info($rrd);
 		for my $key (keys %$info) {
@@ -137,17 +144,30 @@ sub port_init {
 				next;
 			}
 			if($pl[$n] && $np) {
-				my $p = trim(lc((split(',', $port->{desc}->{$pl[$n]}))[1])) || "all";
+				my $p = trim(lc((split(',', $port->{desc}->{$pl[$n]}))[1])) || "";
+				if(! grep {$_ eq $p} ("tcp", "udp", "tcp6", "udp6")) {
+					logger("$myself: Invalid protocol name '$p' in port '$pl[$n]'.");
+					next;
+				}
+				$cmd = "iptables" . $config->{iptables_wait_lock};
+				if(grep {$_ eq $p} ("tcp6", "udp6")) {
+					if(lc($config->{ipv6_disabled} || "") eq "y") {
+						logger("$myself: IPv6 is explicitly disabled, you shouldn't want to monitor 'tcp6' or 'udp6' protocols.");
+						next;
+					}
+					$cmd = "ip6tables" . $config->{iptables_wait_lock};
+					$p =~ s/6//;
+				}
 				my $conn = trim(lc((split(',', $port->{desc}->{$pl[$n]}))[2]));
 				if($conn eq "in" || $conn eq "in/out") {
-					system("iptables -N monitorix_IN_$n 2>/dev/null");
-					system("iptables -I INPUT -p $p --sport 1024:65535 --dport $np -m conntrack --ctstate NEW,ESTABLISHED,RELATED -j monitorix_IN_$n -c 0 0");
-					system("iptables -I OUTPUT -p $p --sport $np --dport 1024:65535 -m conntrack --ctstate ESTABLISHED,RELATED -j monitorix_IN_$n -c 0 0");
+					system("$cmd -t $table -N monitorix_IN_$n 2>/dev/null");
+					system("$cmd -t $table -I INPUT -p $p --sport 1024:65535 --dport $np -m conntrack --ctstate NEW,ESTABLISHED,RELATED -j monitorix_IN_$n -c 0 0");
+					system("$cmd -t $table -I OUTPUT -p $p --sport $np --dport 1024:65535 -m conntrack --ctstate ESTABLISHED,RELATED -j monitorix_IN_$n -c 0 0");
 				}
 				if($conn eq "out" || $conn eq "in/out") {
-					system("iptables -N monitorix_OUT_$n 2>/dev/null");
-					system("iptables -I INPUT -p $p --sport $np --dport 1024:65535 -m conntrack --ctstate ESTABLISHED,RELATED -j monitorix_OUT_$n -c 0 0");
-					system("iptables -I OUTPUT -p $p --sport 1024:65535 --dport $np -m conntrack --ctstate NEW,ESTABLISHED,RELATED -j monitorix_OUT_$n -c 0 0");
+					system("$cmd -t $table -N monitorix_OUT_$n 2>/dev/null");
+					system("$cmd -t $table -I INPUT -p $p --sport $np --dport 1024:65535 -m conntrack --ctstate ESTABLISHED,RELATED -j monitorix_OUT_$n -c 0 0");
+					system("$cmd -t $table -I OUTPUT -p $p --sport 1024:65535 --dport $np -m conntrack --ctstate NEW,ESTABLISHED,RELATED -j monitorix_OUT_$n -c 0 0");
 				}
 				if($conn ne "in" && $conn ne "out" && $conn ne "in/out") {
 					logger("$myself: Invalid connection type '$conn'; must be 'in', 'out' or 'in/out'.");
@@ -164,6 +184,7 @@ sub port_init {
 			if($pl[$n] && $np) {
 				my $p = lc((split(',', $port->{desc}->{$pl[$n]}))[1]) || "all";
 				# in/out not supported yet  FIXME
+				$p =~ s/6//;	# tcp6, udp6, ... not supported
 				system("ipfw -q add $port->{rule} count $p from me $np to any");
 				system("ipfw -q add $port->{rule} count $p from any to me $np");
 			}
@@ -186,26 +207,41 @@ sub port_update {
 	my @i_out;
 	my @o_in;
 	my @o_out;
+	my $table = $config->{ip_default_table};
 
 	my $n;
 	my $rrdata = "N";
 
 	if($config->{os} eq "Linux") {
-		open(IN, "iptables -nxvL INPUT |");
-		while(<IN>) {
+		my @data;
+		my $l;
+		my $cmd;
+		my $cmd6;
+
+		$cmd = "iptables" . $config->{iptables_wait_lock};
+		$cmd6 = "ip6tables" . $config->{iptables_wait_lock};
+		open(IN, "$cmd -t $table -nxvL INPUT 2>/dev/null |");
+		@data = <IN>;
+		close(IN);
+		if(lc($config->{ipv6_disabled} || "") ne "y") {
+			open(IN, "$cmd6 -t $table -nxvL INPUT 2>/dev/null |");
+			push(@data, <IN>);
+			close(IN);
+		}
+		for($l = 0; $l < scalar(@data); $l++) {
 			for($n = 0; $n < $port->{max}; $n++) {
 				$i_in[$n] = 0 unless $i_in[$n];
 				$o_in[$n] = 0 unless $o_in[$n];
-				if(/ monitorix_IN_$n /) {
-					my (undef, $bytes) = split(' ', $_);
+				if($data[$l] =~ / monitorix_IN_$n /) {
+					my (undef, $bytes) = split(' ', $data[$l]);
 					chomp($bytes);
 					$i_in[$n] = $bytes - ($config->{port_hist_i_in}[$n] || 0);
 					$i_in[$n] = 0 unless $i_in[$n] != $bytes;
 					$config->{port_hist_i_in}[$n] = $bytes;
 					$i_in[$n] /= 60;
 				}
-				if(/ monitorix_OUT_$n /) {
-					my (undef, $bytes) = split(' ', $_);
+				if($data[$l] =~ / monitorix_OUT_$n /) {
+					my (undef, $bytes) = split(' ', $data[$l]);
 					chomp($bytes);
 					$o_in[$n] = $bytes - ($config->{port_hist_o_in}[$n] || 0);
 					$o_in[$n] = 0 unless $o_in[$n] != $bytes;
@@ -214,22 +250,28 @@ sub port_update {
 				}
 			}
 		}
+		open(IN, "$cmd -t $table -nxvL OUTPUT 2>/dev/null |");
+		@data = <IN>;
 		close(IN);
-		open(IN, "iptables -nxvL OUTPUT |");
-		while(<IN>) {
+		if(lc($config->{ipv6_disabled} || "") ne "y") {
+			open(IN, "$cmd6 -t $table -nxvL OUTPUT 2>/dev/null |");
+			push(@data, <IN>);
+			close(IN);
+		}
+		for($l = 0; $l < scalar(@data); $l++) {
 			for($n = 0; $n < $port->{max}; $n++) {
 				$o_out[$n] = 0 unless $o_out[$n];
 				$i_out[$n] = 0 unless $i_out[$n];
-				if(/ monitorix_OUT_$n /) {
-					my (undef, $bytes) = split(' ', $_);
+				if($data[$l] =~ / monitorix_OUT_$n /) {
+					my (undef, $bytes) = split(' ', $data[$l]);
 					chomp($bytes);
 					$o_out[$n] = $bytes - ($config->{port_hist_o_out}[$n] || 0);
 					$o_out[$n] = 0 unless $o_out[$n] != $bytes;
 					$config->{port_hist_o_out}[$n] = $bytes;
 					$o_out[$n] /= 60;
 				}
-				if(/ monitorix_IN_$n /) {
-					my (undef, $bytes) = split(' ', $_);
+				if($data[$l] =~ / monitorix_IN_$n /) {
+					my (undef, $bytes) = split(' ', $data[$l]);
 					chomp($bytes);
 					$i_out[$n] = $bytes - ($config->{port_hist_i_out}[$n] || 0);
 					$i_out[$n] = 0 unless $i_out[$n] != $bytes;
@@ -238,7 +280,6 @@ sub port_update {
 				}
 			}
 		}
-		close(IN);
 	}
 	if(grep {$_ eq $config->{os}} ("FreeBSD", "OpenBSD", "NetBSD")) {
 		my @pl = split(',', $port->{list});
@@ -283,6 +324,7 @@ sub port_update {
 
 sub port_cgi {
 	my ($package, $config, $cgi) = @_;
+	my @output;
 
 	my $port = $config->{port};
 	my $tf = $cgi->{tf};
@@ -290,14 +332,23 @@ sub port_cgi {
 	my $graph = $cgi->{graph};
 	my $silent = $cgi->{silent};
 	my $zoom = "--zoom=" . $config->{global_zoom};
+	my %rrd = (
+		'new' => \&RRDs::graphv,
+		'old' => \&RRDs::graph,
+	);
+	my $version = "new";
+	my $pic;
+	my $picz;
+	my $picz_width;
+	my $picz_height;
 
 	my $u = "";
 	my $width;
 	my $height;
 	my @riglim;
 	my @warning;
-	my @PNG;
-	my @PNGz;
+	my @IMG;
+	my @IMGz;
 	my $name;
 	my @tmp;
 	my @tmpz;
@@ -310,9 +361,12 @@ sub port_cgi {
 	my $str;
 	my $err;
 
+	$version = "old" if $RRDs::VERSION < 1.3;
 	my $rrd = $config->{base_lib} . $package . ".rrd";
 	my $title = $config->{graph_title}->{$package};
-	my $PNG_DIR = $config->{base_dir} . "/" . $config->{imgs_dir};
+	my $IMG_DIR = $config->{base_dir} . "/" . $config->{imgs_dir};
+	my $imgfmt_uc = uc($config->{image_format});
+	my $imgfmt_lc = lc($config->{image_format});
 
 	$title = !$silent ? $title : "";
 
@@ -326,34 +380,34 @@ sub port_cgi {
 	#
 	if(lc($config->{iface_mode}) eq "text") {
 		if($title) {
-			main::graph_header($title, 2);
-			print("    <tr>\n");
-			print("    <td bgcolor='$colors->{title_bg_color}'>\n");
+			push(@output, main::graph_header($title, 2));
+			push(@output, "    <tr>\n");
+			push(@output, "    <td bgcolor='$colors->{title_bg_color}'>\n");
 		}
 		my (undef, undef, undef, $data) = RRDs::fetch("$rrd",
 			"--start=-$tf->{nwhen}$tf->{twhen}",
 			"AVERAGE",
 			"-r $tf->{res}");
 		$err = RRDs::error;
-		print("ERROR: while fetching $rrd: $err\n") if $err;
+		push(@output, "ERROR: while fetching $rrd: $err\n") if $err;
 		my $line1;
 		my $line2;
-		print("    <pre style='font-size: 12px; color: $colors->{fg_color}';>\n");
-		print("    ");
+		push(@output, "    <pre style='font-size: 12px; color: $colors->{fg_color}';>\n");
+		push(@output, "    ");
 		my $max = min($port->{max}, scalar(my @pl = split(',', $port->{list})));
 		for($n = 0; $n < $max; $n++) {
 			$pl[$n] = trim($pl[$n]);
 			my $pn = trim((split(',', $port->{desc}->{$pl[$n]}))[0]);
 			my $pc = trim((split(',', $port->{desc}->{$pl[$n]}))[2]);
 			foreach(split('/', $pc)) {
-				printf("   %-5s %10s", $pl[$n], uc(trim($_)) . "-" . $pn);
+				push(@output, sprintf("   %-5s %10s", $pl[$n], uc(trim($_)) . "-" . $pn));
 				$line1 .= "    K$T/s_I   K$T/s_O";
 				$line2 .= "-------------------";
 			}
 		}
-		print("\n");
-		print("Time$line1\n");
-		print("----$line2 \n");
+		push(@output, "\n");
+		push(@output, "Time$line1\n");
+		push(@output, "----$line2 \n");
 		my $line;
 		my @row;
 		my $time;
@@ -363,7 +417,7 @@ sub port_cgi {
 		for($n = 0, $time = $tf->{tb}; $n < ($tf->{tb} * $tf->{ts}); $n++) {
 			$line = @$data[$n];
 			$time = $time - (1 / $tf->{ts});
-			printf(" %2d$tf->{tc} ", $time);
+			push(@output, sprintf(" %2d$tf->{tc} ", $time));
 			for($n2 = 0; $n2 < $max; $n2++) {
 				$pl[$n2] = trim($pl[$n2]);
 				my $pc = trim((split(',', $port->{desc}->{$pl[$n2]}))[2]);
@@ -384,24 +438,24 @@ sub port_cgi {
 				foreach(split('/', $pc)) {
 					if(lc($_) eq "in") {
 						@row = ($k_i_in, $k_i_out);
-						printf("   %6d   %6d ", @row);
+						push(@output, sprintf("   %6d   %6d ", @row));
 					}
 					if(lc($_) eq "out") {
 						@row = ($k_o_in, $k_o_out);
-						printf("   %6d   %6d ", @row);
+						push(@output, sprintf("   %6d   %6d ", @row));
 					}
 				}
 			}
-			print("\n");
+			push(@output, "\n");
 		}
-		print("    </pre>\n");
+		push(@output, "    </pre>\n");
 		if($title) {
-			print("    </td>\n");
-			print("    </tr>\n");
-			main::graph_footer();
+			push(@output, "    </td>\n");
+			push(@output, "    </tr>\n");
+			push(@output, main::graph_footer());
 		}
-		print("  <br>\n");
-		return;
+		push(@output, "  <br>\n");
+		return @output;
 	}
 
 
@@ -423,13 +477,13 @@ sub port_cgi {
 
 		my $pc = trim((split(',', $port->{desc}->{$pl[$n]}))[2]);
 		foreach my $conn (split('/', $pc)) {
-			$str = $u . $package . $n . substr($conn, 0 ,1) . "." . $tf->{when} . ".png";
-			push(@PNG, $str);
-			unlink("$PNG_DIR" . $str);
+			$str = $u . $package . $n . substr($conn, 0 ,1) . "." . $tf->{when} . ".$imgfmt_lc";
+			push(@IMG, $str);
+			unlink("$IMG_DIR" . $str);
 			if(lc($config->{enable_zoom}) eq "y") {
-				$str = $u . $package . $n . substr($conn, 0 ,1) . "z." . $tf->{when} . ".png";
-				push(@PNGz, $str);
-				unlink("$PNG_DIR" . $str);
+				$str = $u . $package . $n . substr($conn, 0 ,1) . "z." . $tf->{when} . ".$imgfmt_lc";
+				push(@IMGz, $str);
+				unlink("$IMG_DIR" . $str);
 			}
 		}
 	}
@@ -439,7 +493,7 @@ sub port_cgi {
 	while($n < $max) {
 		next unless $pl[$n];
 
-		# continue if port was listed by not defined
+		# continue if port was listed but not defined
 		if(!$port->{desc}->{$pl[$n]}) {
 			$n++;
 			next;
@@ -447,65 +501,64 @@ sub port_cgi {
 
 		if($title) {
 			if($n == 0) {
-				main::graph_header($title, $port->{graphs_per_row});
+				push(@output, main::graph_header($title, $port->{graphs_per_row}));
 			}
 			if($n2 == 1) {
-				print("    <tr>\n");
+				push(@output, "    <tr>\n");
 			}
 		}
 
 		my $pc = trim((split(',', $port->{desc}->{$pl[$n]}))[2]);
 		foreach my $pcon (split('/', $pc)) {
 			if($title) {
-				print("    <td bgcolor='" . $colors->{title_bg_color} . "'>\n");
+				push(@output, "    <td bgcolor='" . $colors->{title_bg_color} . "'>\n");
 			}
 			my $pnum;
 			$pl[$n] = trim($pl[$n]);
+			my $num = ($pl[$n] =~ m/^(\d+).*?/)[0];	# strips any suffix from port number
 			my $pn = trim((split(',', $port->{desc}->{$pl[$n]}))[0]);
 			my $pp = trim((split(',', $port->{desc}->{$pl[$n]}))[1]);
+			$pp =~ s/6//;
 			my $prig = trim((split(',', $port->{desc}->{$pl[$n]}))[3]);
 			my $plim = trim((split(',', $port->{desc}->{$pl[$n]}))[4]);
-			undef(@riglim);
-			if(trim($prig) eq 1) {
-				push(@riglim, "--upper-limit=" . trim($plim));
-			} else {
-				if(trim($prig) eq 2) {
-					push(@riglim, "--upper-limit=" . trim($plim));
-					push(@riglim, "--rigid");
-				}
-			}
+			my $plis = trim((split(',', $port->{desc}->{$pl[$n]}))[5]);
+			@riglim = @{setup_riglim($prig, $plim)};
+
+			# check if the network port is still listening
 			undef(@warning);
-			if($config->{os} eq "Linux") {
-				open(IN, "netstat -nl --$pp |");
-				while(<IN>) {
-					(undef, undef, undef, $pnum) = split(' ', $_);
-					chomp($pnum);
-					$pnum =~ s/.*://;
-					if($pnum eq $pl[$n]) {
-						last;
-					}
-				}
-				close(IN);
-			}
-			if(grep {$_ eq $config->{os}} ("FreeBSD", "OpenBSD")) {
-				open(IN, "netstat -anl -p $pp |");
-				while(<IN>) {
-					 my $stat;
-					(undef, undef, undef, $pnum, undef, $stat) = split(' ', $_);
-					chomp($stat);
-					if($stat eq "LISTEN") {
+			if(uc($plis) eq "L") {
+				if($config->{os} eq "Linux") {
+					open(IN, "netstat -nl --$pp |");
+					while(<IN>) {
+						(undef, undef, undef, $pnum) = split(' ', $_);
 						chomp($pnum);
-						($pnum) = ($pnum =~ m/^.*?(\.\d+$)/);
-						$pnum =~ s/\.//;
-						if($pnum eq $pl[$n]) {
+						$pnum =~ s/.*://;
+						if($pnum eq $num) {
 							last;
 						}
 					}
+					close(IN);
+				}
+				if(grep {$_ eq $config->{os}} ("FreeBSD", "OpenBSD")) {
+					open(IN, "netstat -anl -p $pp |");
+					while(<IN>) {
+					 	my $stat;
+						(undef, undef, undef, $pnum, undef, $stat) = split(' ', $_);
+						chomp($stat);
+						if($stat eq "LISTEN") {
+							chomp($pnum);
+							($pnum) = ($pnum =~ m/^.*?(\.\d+$)/);
+							$pnum =~ s/\.//;
+							if($pnum eq $num) {
+								last;
+							}
+						}
+					}
+					close(IN);
+				}
+				if(lc($pcon) ne "out" && $pnum ne $num) {
+					push(@warning, $colors->{warning_color});
 				}
-				close(IN);
-			}
-			if($pnum ne $pl[$n]) {
-				push(@warning, $colors->{warning_color});
 			}
 
 			$name = substr(uc($pcon) . "-" . $pn, 0, 15);
@@ -567,15 +620,14 @@ sub port_cgi {
 				push(@tmp, "COMMENT: \\n");
 				push(@tmp, "COMMENT: \\n");
 			}
-			RRDs::graph("$PNG_DIR" . "$PNG[$n3]",
+			$pic = $rrd{$version}->("$IMG_DIR" . "$IMG[$n3]",
 				"--title=$name traffic  ($tf->{nwhen}$tf->{twhen})",
 				"--start=-$tf->{nwhen}$tf->{twhen}",
-				"--imgformat=PNG",
+				"--imgformat=$imgfmt_uc",
 				"--vertical-label=$vlabel",
 				"--width=$width",
 				"--height=$height",
 				@riglim,
-				"--lower-limit=0",
 				$zoom,
 				@{$cgi->{version12}},
 				@{$cgi->{version12_small}},
@@ -589,18 +641,18 @@ sub port_cgi {
 				@CDEF,
 				@tmp);
 			$err = RRDs::error;
-			print("ERROR: while graphing $PNG_DIR" . "$PNG[$n3]: $err\n") if $err;
+			push(@output, "ERROR: while graphing $IMG_DIR" . "$IMG[$n3]: $err\n") if $err;
 			if(lc($config->{enable_zoom}) eq "y") {
 				($width, $height) = split('x', $config->{graph_size}->{zoom});
-				RRDs::graph("$PNG_DIR" . "$PNGz[$n3]",
+				$picz = $rrd{$version}->("$IMG_DIR" . "$IMGz[$n3]",
 					"--title=$name traffic  ($tf->{nwhen}$tf->{twhen})",
 					"--start=-$tf->{nwhen}$tf->{twhen}",
-					"--imgformat=PNG",
+					"--imgformat=$imgfmt_uc",
 					"--vertical-label=$vlabel",
 					"--width=$width",
 					"--height=$height",
 					@riglim,
-					"--lower-limit=0",
+					$zoom,
 					@{$cgi->{version12}},
 					@{$cgi->{version12_small}},
 					@{$colors->{graph_colors}},
@@ -613,29 +665,35 @@ sub port_cgi {
 					@CDEF,
 					@tmpz);
 				$err = RRDs::error;
-				print("ERROR: while graphing $PNG_DIR" . "$PNGz[$n3]: $err\n") if $err;
+				push(@output, "ERROR: while graphing $IMG_DIR" . "$IMGz[$n3]: $err\n") if $err;
 			}
 			if($title || ($silent =~ /imagetag/ && $graph =~ /port$n3/)) {
 				if(lc($config->{enable_zoom}) eq "y") {
 					if(lc($config->{disable_javascript_void}) eq "y") {
-						print("      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $PNGz[$n3] . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$n3] . "' border='0'></a>\n");
-					}
-					else {
-						print("      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $PNGz[$n3] . "','','width=" . ($width + 115) . ",height=" . ($height + 100) . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$n3] . "' border='0'></a>\n");
+						push(@output, "      <a href=\"" . $config->{url} . "/" . $config->{imgs_dir} . $IMGz[$n3] . "\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$n3] . "' border='0'></a>\n");
+					} else {
+						if($version eq "new") {
+							$picz_width = $picz->{image_width} * $config->{global_zoom};
+							$picz_height = $picz->{image_height} * $config->{global_zoom};
+						} else {
+							$picz_width = $width + 115;
+							$picz_height = $height + 100;
+						}
+						push(@output, "      <a href=\"javascript:void(window.open('" . $config->{url} . "/" . $config->{imgs_dir} . $IMGz[$n3] . "','','width=" . $picz_width . ",height=" . $picz_height . ",scrollbars=0,resizable=0'))\"><img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$n3] . "' border='0'></a>\n");
 					}
 				} else {
-					print("      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $PNG[$n3] . "'>\n");
+					push(@output, "      <img src='" . $config->{url} . "/" . $config->{imgs_dir} . $IMG[$n3] . "'>\n");
 				}
 			}
 			if($title) {
-				print("    </td>\n");
+				push(@output, "    </td>\n");
 			}
 
 			if($n2 < $port->{graphs_per_row} && $n2 < $max) {
 				$n2++;
 			} else {
 				if($title) {
-					print("    </tr>\n");
+					push(@output, "    </tr>\n");
 				}
 				$n2 = 1;
 			}
@@ -644,10 +702,10 @@ sub port_cgi {
 		$n++;
 	}
 	if($title) {
-		main::graph_footer();
+		push(@output, main::graph_footer());
 	}
-	print("  <br>\n");
-	return;
+	push(@output, "  <br>\n");
+	return @output;
 }
 
 1;
