@@ -1,7 +1,7 @@
 #
 # Monitorix - A lightweight system monitoring tool.
 #
-# Copyright (C) 2005-2017 by Jordi Sanfeliu <jordi@fibranet.cat>
+# Copyright (C) 2005-2019 by Jordi Sanfeliu <jordi@fibranet.cat>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -122,7 +122,7 @@ sub system_init {
 	}
 
 	# check dependencies
-	if(lc($system->{alerts}->{loadavg_enabled}) eq "y") {
+	if(lc($system->{alerts}->{loadavg_enabled} || "") eq "y") {
 		if(! -x $system->{alerts}->{loadavg_script}) {
 			logger("$myself: ERROR: script '$system->{alerts}->{loadavg_script}' doesn't exist or don't has execution permissions.");
 		}
@@ -167,6 +167,8 @@ sub system_update {
 	my $entropy = 0;
 	my $uptime = 0;
 
+	my $srecl = 0;
+	my $snorecl = 0;
 	my $rrdata = "N";
 
 	if($config->{os} eq "Linux") {
@@ -231,11 +233,24 @@ sub system_update {
 			}
 			if(/^Inactive:\s+(\d+) kB$/) {
 				$minac = $1;
+				next;
+			}
+			if(/^SReclaimable:\s+(\d+) kB$/) {
+				$srecl = $1;
+				next;
+			}
+			if(/^SUnreclaim:\s+(\d+) kB$/) {
+				$snorecl = $1;
 				last;
 			}
 		}
 		close(IN);
 
+		# SReclaimable and SUnreclaim values are added to 'mfree'
+		# in order to be also included in the subtraction later.
+		$mfree += $srecl;
+		$mfree += $snorecl;
+
 		open(IN, "/proc/sys/kernel/random/entropy_avail");
 		while(<IN>) {
 			if(/^(\d+)$/) {
@@ -290,10 +305,10 @@ sub system_update {
 		$macti = ($page_size * $macti) / 1024;
 		$minac = ($page_size * $minac) / 1024;
 
-		open(IN, "/sbin/sysctl -n kern.random.sys.seeded |");
-		$entropy = <IN>;
-		close(IN);
-		chomp($entropy);
+#		open(IN, "/sbin/sysctl -n kern.random.sys.seeded |");
+#		$entropy = <IN>;
+#		close(IN);
+#		chomp($entropy);
 
 		open(IN, "/sbin/sysctl -n kern.boottime |");
 		(undef, undef, undef, $uptime) = split(' ', <IN>);
@@ -380,7 +395,10 @@ sub system_update {
 
 	# SYSTEM alert
 	if(lc($system->{alerts}->{loadavg_enabled}) eq "y") {
-		if(!$system->{alerts}->{loadavg_threshold} || $load15 < $system->{alerts}->{loadavg_threshold}) {
+		my $load;
+
+		$load = min($load5, $load15);
+		if(!$system->{alerts}->{loadavg_threshold} || $load < $system->{alerts}->{loadavg_threshold}) {
 			$config->{system_hist_alert1} = 0;
 		} else {
 			if(!$config->{system_hist_alert1}) {
@@ -389,7 +407,7 @@ sub system_update {
 			if($config->{system_hist_alert1} > 0 && (time - $config->{system_hist_alert1}) >= $system->{alerts}->{loadavg_timeintvl}) {
 				if(-x $system->{alerts}->{loadavg_script}) {
 					logger("$myself: ALERT: executing script '$system->{alerts}->{loadavg_script}'.");
-					system($system->{alerts}->{loadavg_script} . " " .$system->{alerts}->{loadavg_timeintvl} . " " . $system->{alerts}->{loadavg_threshold} . " " . $load15);
+					system($system->{alerts}->{loadavg_script} . " " .$system->{alerts}->{loadavg_timeintvl} . " " . $system->{alerts}->{loadavg_threshold} . " " . $load);
 				} else {
 					logger("$myself: ERROR: script '$system->{alerts}->{loadavg_script}' doesn't exist or don't has execution permissions.");
 				}
@@ -489,7 +507,7 @@ sub system_cgi {
 			$buff /= 1024;
 			$cach /= 1024;
 			$free /= 1024;
-			@row = ($load1, $load5, $load15, $total_mem - $free, $cach, $buff, $macti, $minac, $nproc, $nprun, $npslp, $npwio, $npzom, $npstp, $npswp, $entropy, $uptime);
+			@row = ($load1, $load5, $load15, ($total_mem - $free - $buff - $cach), $cach, $buff, $macti, $minac, $nproc, $nprun, $npslp, $npwio, $npzom, $npstp, $npswp, $entropy, $uptime);
 			$time = $time - (1 / $tf->{ts});
 			push(@output, sprintf(" %2d$tf->{tc}   %4.1f  %4.1f  %4.1f  %6d  %6d  %6d  %6d  %6d  %4d %4d %4d %4d %4d %4d %4d  %6d %8d\n", $time, @row));
 		}
@@ -707,7 +725,7 @@ sub system_cgi {
 		"CDEF:m_mtotl=mtotl,1024,*",
 		"CDEF:m_mbuff=mbuff,1024,*",
 		"CDEF:m_mcach=mcach,1024,*",
-		"CDEF:m_mused=m_mtotl,mfree,1024,*,-",
+		"CDEF:m_mused=m_mtotl,mfree,1024,*,-,m_mbuff,-,m_mcach,-",
 		"CDEF:m_macti=macti,1024,*",
 		"CDEF:m_minac=minac,1024,*",
 		"CDEF:allvalues=mtotl,mbuff,mcach,mfree,macti,minac,+,+,+,+,+",
@@ -742,7 +760,7 @@ sub system_cgi {
 			"CDEF:m_mtotl=mtotl,1024,*",
 			"CDEF:m_mbuff=mbuff,1024,*",
 			"CDEF:m_mcach=mcach,1024,*",
-			"CDEF:m_mused=m_mtotl,mfree,1024,*,-",
+			"CDEF:m_mused=m_mtotl,mfree,1024,*,-,m_mbuff,-,m_mcach,-",
 			"CDEF:m_macti=macti,1024,*",
 			"CDEF:m_minac=minac,1024,*",
 			"CDEF:allvalues=mtotl,mbuff,mcach,mfree,macti,minac,+,+,+,+,+",
